
#
# trace.py                                                             
#                                                                      
# Copyright (C) 2001 Gerson Kurz
# 
# This program is free software; you can redistribute it and/or modify 
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation; either version 1, or (at your option)  
# any later version.                                                   
#                                                                      
# This program is distributed in the hope that it will be useful,      
# but WITHOUT ANY WARRANTY; without even the implied warranty of       
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
# GNU General Public License for more details.                         
#                                                                      
# You should have received a copy of the GNU General Public License    
# along with this program; if not, write to the Free Software          
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            
#

import os, sys, traceback, time, threading

# external variables
if os.name == 'nt':
    TraceFileName = "c:\\pergamon.log"
else:
    TraceFileName = "/var/log/pergamon.log"
MaxTraceFileSize = 0 # 1MB
BackupOldTracefiles = 3

# internal variables
__FirstTime = 1
old_stdout = sys.stdout
g_bOutputToConsole = 1

TraceMutex = threading.Semaphore()

class TraceWriter:
    "This class is used for writing data to the tracefile"
    def __init__(self,handle):
        self.handle = handle
        self.SizeUsed = 0
        if os.name == "nt":
            self._write = self._write_nt
        else:
            self._write = self._write_posix

    def write(self,message):
        message = message.strip()
        if not message:
            return

        global TraceMutex
        TraceMutex.acquire(blocking=1)
        try:        
            frame = sys._getframe(1)
            global g_bOutputToConsole
            if g_bOutputToConsole:
                old_stdout.write(message+'\n')
            header = '%s %s: ' % (time.ctime(),threading.currentThread().getName())
            self._write(header + message+'\n')
        except:
            pass
        TraceMutex.release()

    def _write_nt(self,message):
        "nt-specific write function that replaces \n by \r\n"
        if MaxTraceFileSize:
            message = message.replace('\r','')
            message = message.replace('\n','\r\n')
            l = len(message)
            if self.SizeUsed+l > MaxTraceFileSize:
                top = MaxTraceFileSize-self.SizeUsed
                self.handle.write(message[:top])
                self.handle.seek(0)
                self.handle.write(message[top:])
                self.SizeUsed = l-top
                self.handle.flush()
                return
            self.SizeUsed += l
        self.handle.write(message)
        self.handle.flush()
        
    def _write_posix(self,message):
        "posix-specific trace function"
        if MaxTraceFileSize:
            l = len(message)
            if self.SizeUsed+l > MaxTraceFileSize:
                top = MaxTraceFileSize-self.SizeUsed
                self.handle.write(message[:top])
                self.handle.seek(0)
                self.handle.write(message[top:])
                self.SizeUsed = l-top
                self.handle.flush()
                return
            self.SizeUsed += l
        self.handle.write(message)
        self.handle.flush()

    def close(self):
        self.handle.close()
        sys.stdout = old_stdout

def BackupTracefiles(filename,count):
    "Maintains a backup of old tracefiles"
    deleteme = filename + ".%d" % (count-1)
    try:
        os.unlink(deleteme)
    except:
        pass
    for i in range(count-2,-1,-1):
        oldname = filename + ".%d" % i
        newname = filename + ".%d" % (i+1)
        try:
            os.rename(oldname,newname)
        except:
            pass
    try:
        os.rename(filename,oldname)
    except:
        pass

def Enable(bEnable=1,bOutputToConsole=1):
    global g_bOutputToConsole
    g_bOutputToConsole = bOutputToConsole
    "Called to globally enable or disable tracing"
    try:
        global __FirstTime
        if bEnable:
            if __FirstTime:
                __FirstTime = 0
                BackupTracefiles(TraceFileName,BackupOldTracefiles)
            tw = TraceWriter(open(TraceFileName,"wb"))
            tw.stdout = sys.stdout
            sys.stdout = tw
            sys.stderr = sys.stdout
        else:
            if sys.stdout != old_stdout:
                sys.stdout.close()
            sys.stdout = old_stdout
            
        return 1
    except:
        traceback.print_exc()
        return 0
